% Bandpass Filter for Michael T. Owyang and Tatevik Sekhposyan:
% "Okun's Law of the Business Cycle: Was the Great Recession All THat Different?"
% Federal Reserve Bank of St. Louis Review, October 2012

%    bpass.m
%    program to compute bandpass filtered series using upper and lower
%    cutoff periods.


function xf = bpass(x,updc,lpdc,n)

%	 INPUT :
%      x = series to be filtered
%      updc = Period corresponding to upper cutoff frequency
%      lpdc = Period corresponding to lower cutoff frequency
%      n = number of terms in moving average filter

%      Note: updc=2 makes this a high pass filter
%      (since period = 2 implies om=pi).

%      if lpdc .> size(x,1), frequency is set to zero



disp('Band Pass Filtering of Time Series: ');
disp(['Components between Periods of ',num2str(updc),' and ', num2str(lpdc), ' Time Units']);


% Implied Frequencies %
omubar=2*pi/updc;
omlbar=2*pi/lpdc;
if (lpdc >= size(x,1)); omlbar=0; end;


 %To construct a low pass filter, with a cutoff frequency of "ombar",
 %we note that the transfer function of the approximating filter
 %is given by:

 % alpha(om) = a0 + a1 cos(om) + ... aK cos(K om)

 %and the ak's are given by:

 %a0 = ombar/(pi)

 %ak = sin(k ombar)/(k pi)

 %where ombar is the cutoff frequency.

 % We employ the fact that a bandpass filter is the difference between two
 % low pass filters,
 %   bp(L) = bu(L) - bl(L)
 % with bu(L) being the filter with the high cutoff point and bl(L) being
 % that with the low cutoff point.
 

% Define the vector of K's to be studied %

% Set the grid for om %
step=.01; np=(2/step)+1;
om=1:step:np;

om=pi*om;
omp=om/pi;

% Initialize output matrix %

%  Loop over specified K's %

 %  Construct Filter Weights %

 akvec=zeros(n+1,1);

 akvec(1)=(omubar-omlbar)/(pi);

 kk=1; while kk <= n;
  akvec(kk+1)=(sin(kk*omubar)-sin(kk*omlbar))/(kk*pi);
 kk=kk+1; end;

 
  %Impose constraint that transfer is
  %  (i)  0 at om = 0 if oml .> 0;
  %  (ii) 1 at om = 0 if oml .== 0;
  %This amounts to requiring that weights sum to zero.
  %Initial sum of weights:
 
lam=akvec(1)+2*sum(akvec(2:n+1),1);
% amount to add to each weight to get sum to add to zero  
if (omlbar > .00000001);
   lam = -lam/(2*(n+1));
 else;
   lam = (1-lam)/(2*(n+1));
end;
akvec=akvec+lam;
akvec(1)=akvec(1)+lam;

t=size(x,1);    % number of observations %

% Set vector of weights %

avec=zeros(2*n+1,1);
avec(n+1)=akvec(1);

i=1; while i <= n;
 avec(n+1-i)=akvec(i+1);
 avec(n+1+i)=akvec(i+1);
i=i+1; end;

xf=zeros(t,1);
j=n+1; while j <= t-n;
 xf(j)=x(j-n:j+n)'*avec;
j=j+1; end;